@@ -1,7 +1,7 @@
 /* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
 /*
 **
-** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2016 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -117,28 +117,6 @@ mu_util_dir_expand (const char *path)
 	return g_strdup (resolved);
 }
 
-
-char*
-mu_util_create_tmpdir (void)
-{
-	gchar *dirname;
-
-	dirname =  g_strdup_printf ("%s%cmu-%d%c%x",
-				    g_get_tmp_dir(),
-				    G_DIR_SEPARATOR,
-				    getuid(),
-				    G_DIR_SEPARATOR,
-				    (int)random()*getpid()*(int)time(NULL));
-
-	if (!mu_util_create_dir_maybe (dirname, 0700, FALSE)) {
-		g_free (dirname);
-		return NULL;
-	}
-
-	return dirname;
-}
-
-
 GQuark
 mu_util_error_quark (void)
 {
@@ -221,9 +199,16 @@ gchar*
 mu_util_guess_mu_homedir (void)
 {
 	const char* home;
+	const gchar *hdir1;
 
-	/* g_get_home_dir use /etc/passwd, not $HOME; this is better,
-	 * as HOME may be wrong when using 'sudo' etc.*/
+	/* first, try MU_HOME */
+	hdir1 = g_getenv ("MU_HOME");
+
+	if (hdir1 && mu_util_check_dir (hdir1, TRUE, FALSE))
+		return g_strdup (hdir1);
+
+	/* then, g_get_home_dir use /etc/passwd, not $HOME; this is
+	 * better, as HOME may be wrong when using 'sudo' etc.*/
 	home = g_get_home_dir ();
 
 	if (!home) {
@@ -264,31 +249,6 @@ mu_util_create_dir_maybe (const gchar *path, mode_t mode, gboolean nowarn)
 }
 
 
-gchar*
-mu_util_str_from_strv (const gchar **params)
-{
-	GString *str;
-	int i;
-
-	g_return_val_if_fail (params, NULL);
-
-	if (!params[0])
-		return g_strdup ("");
-
-	str = g_string_sized_new (64); /* just a guess */
-
-	for (i = 0; params[i]; ++i) {
-
-		if (i > 0)
-			g_string_append_c (str, ' ');
-
-		g_string_append (str, params[i]);
-	}
-
-	return g_string_free (str, FALSE);
-}
-
-
 int
 mu_util_create_writeable_fd (const char* path, mode_t mode,
 			     gboolean overwrite)
@@ -352,6 +312,17 @@ mu_util_program_in_path (const char *prog)
 }
 
 
+/*
+ * Set the child to a group leader to avoid being killed when the
+ * parent group is killed.
+ */
+static void
+maybe_setsid (G_GNUC_UNUSED gpointer user_data)
+{
+#if HAVE_SETSID
+	setsid();
+#endif /*HAVE_SETSID*/
+}
 
 gboolean
 mu_util_play (const char *path, gboolean allow_local, gboolean allow_remote,
@@ -388,8 +359,8 @@ mu_util_play (const char *path, gboolean allow_local, gboolean allow_remote,
 
 	err = NULL;
 	rv = g_spawn_async (NULL, (gchar**)&argv, NULL,
-			    G_SPAWN_SEARCH_PATH, NULL, NULL, NULL,
-			    err);
+			    G_SPAWN_SEARCH_PATH, maybe_setsid,
+			    NULL, NULL, err);
 	return rv;
 }
 
@@ -482,6 +453,7 @@ mu_util_g_set_error (GError **err, MuError errcode, const char *frm, ...)
 }
 
 
+
 static gboolean
 print_args (FILE *stream, const char *frm, va_list args)
 {
