@@ -1,6 +1,6 @@
 ;; mu4e-draft.el -- part of mu4e, the mu mail user agent for emacs
 ;;
-;; Copyright (C) 2011-2012 Dirk-Jan C. Binnema
+;; Copyright (C) 2011-2016 Dirk-Jan C. Binnema
 
 ;; Author: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 ;; Maintainer: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
@@ -53,12 +53,10 @@ messages. This is the mu4e-specific version of
   :group 'mu4e-compose)
 
 (defcustom mu4e-compose-signature
-  (or (and (stringp message-signature) message-signature)
-    "Sent with my mu4e")
+  (or message-signature "Sent with my mu4e")
   "The message signature (i.e. the blob at the bottom of
 messages). This is the mu4e-specific version of
 `message-signature'."
-  :type 'sexp
   :group 'mu4e-compose)
 
 (defcustom mu4e-compose-signature-auto-include t
@@ -73,16 +71,20 @@ message; if nil, only do so when sending the message."
   :type 'boolean
   :group 'mu4e-compose)
 
-(defun mu4e~draft-user-agent-construct ()
-  "Return the User-Agent string for mu4e.
-This is either the value of `mu4e-user-agent', or, if not set, a
-string based on the versions of mu4e and emacs."
-  (format "mu4e %s; emacs %s" mu4e-mu-version emacs-version))
+(defcustom mu4e-compose-in-new-frame nil
+  "Whether to compose messages in a new frame instead of the
+current window."
+  :type 'boolean
+  :group 'mu4e-compose)
+
+(defvar mu4e-user-agent-string
+  (format "mu4e %s; emacs %s" mu4e-mu-version emacs-version)
+  "The User-Agent string for mu4e, or nil.")
 
 (defun mu4e~draft-cite-original (msg)
   "Return a cited version of the original message MSG as a plist.
-This function uses gnus' `mu4e-compose-cite-function', and as such
-all its settings apply."
+This function uses `mu4e-compose-cite-function', and as such all
+its settings apply."
   (with-temp-buffer
     (when (fboundp 'mu4e-view-message-text) ;; keep bytecompiler happy
       (let ((mu4e-view-date-format "%Y-%m-%dT%T%z"))
@@ -97,7 +99,6 @@ all its settings apply."
 	(funcall mu4e-compose-cite-function))
       (pop-mark)
       (goto-char (point-min))
-      (mu4e~fontify-cited)
       (buffer-string))))
 
 (defun mu4e~draft-header (hdr val)
@@ -107,7 +108,17 @@ If VAL is nil, return nil."
   ;; later.
   (when val (format "%s: %s\n"
 	      (propertize hdr 'face 'mu4e-header-key-face)
-	      (propertize val 'face 'mu4e-header-val-face))))
+	      (propertize val 'face 'mu4e-header-value-face))))
+
+(defconst mu4e~max-reference-num 21
+  "Maximum number of References:, as suggested by
+`message-shorten-references'.")
+
+(defun mu4e~shorten-1 (list cut surplus)
+  "Cut SURPLUS elements out of LIST, beginning with CUTth
+one. Code borrowed from `message-shorten-1'."
+  (setcdr (nthcdr (- cut 2) list)
+	  (nthcdr (+ (- cut 2) surplus 1) list)))
 
 (defun mu4e~draft-references-construct (msg)
   "Construct the value of the References: header based on MSG as a
@@ -123,10 +134,16 @@ References. If both are empty, return nil."
 	  (refs (if (and msgid (not (string= msgid "")))
 		  (append refs (list msgid)) refs))
 	  ;; no doubles
-	  (refs (delete-duplicates refs :test #'equal)))
+	  (refs (delete-duplicates refs :test #'equal))
+	  (refnum (length refs))
+	  (cut 2))
+    ;; remove some refs when there are too many
+    (when (> refnum mu4e~max-reference-num)
+      (let ((surplus (- refnum mu4e~max-reference-num)))
+	(mu4e~shorten-1 refs cut surplus)))
     (mapconcat (lambda (id) (format "<%s>" id)) refs " ")))
 
- 
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; determine the recipient fields for new messages
 
@@ -174,6 +191,30 @@ the list form the original."
       reply-to)))
 
 
+(defun mu4e~strip-ignored-addresses (addrs)
+  "Return all the addresses in ADDRS not matching
+`mu4e-compose-reply-ignore-address'."
+  (cond
+    ((null mu4e-compose-reply-ignore-address)
+      addrs)
+    ((functionp mu4e-compose-reply-ignore-address)
+      (remove-if
+	(lambda (elt)
+	  (funcall mu4e-compose-reply-ignore-address (cdr elt)))
+	addrs))
+    (t
+      ;; regexp or list of regexps
+      (let* ((regexp mu4e-compose-reply-ignore-address)
+	     (regexp (if (listp regexp)
+		       (mapconcat (lambda (elt) (concat "\\(" elt "\\)"))
+			 regexp "\\|")
+		       regexp)))
+	(remove-if
+	  (lambda (elt)
+	    (string-match regexp (cdr elt)))
+	  addrs)))))
+
+
 (defun mu4e~draft-create-cc-lst (origmsg reply-all)
   "Create a list of address for the Cc: in a new message, based on
 the original message ORIGMSG, and whether it's a reply-all."
@@ -194,6 +235,8 @@ the original message ORIGMSG, and whether it's a reply-all."
 		      (mu4e~draft-address-cell-equal cc-cell to-cell))
 		    (mu4e~draft-create-to-lst origmsg)))
 		cc-lst))
+	    ;; remove ignored addresses
+	    (cc-lst (mu4e~strip-ignored-addresses cc-lst))
 	    ;; finally, we need to remove ourselves from the cc-list
 	    ;; unless mu4e-compose-keep-self-cc is non-nil
 	    (cc-lst
@@ -229,7 +272,7 @@ based on `user-full-name' and `user-mail-address'; if the latter is
 nil, function returns nil."
   (when user-mail-address
     (if user-full-name
-      (format "%s <%s>" user-full-name user-mail-address)
+      (format "%s <%s>" (mu4e~rfc822-quoteit user-full-name) user-mail-address)
       (format "%s" user-mail-address))))
 
 
@@ -243,8 +286,7 @@ separator is never written to the message file. Also see
 `mu4e-remove-mail-header-separator'."
   ;; we set this here explicitly, since (as it has happened) a wrong
   ;; value for this (such as "") breaks address completion and other things
-  (set (make-local-variable 'mail-header-separator)
-    (purecopy "--text follows this line--"))
+  (set (make-local-variable 'mail-header-separator) "--text follows this line--")
   (put 'mail-header-separator 'permanent-local t)
   (save-excursion
     ;; make sure there's not one already
@@ -260,7 +302,13 @@ separator is never written to the message file. Also see
       ;; search for the first empty line
       (goto-char (point-min))
       (if (search-forward-regexp "^$" nil t)
-	  (replace-match sepa)
+	  (progn
+	    (replace-match sepa)
+	    ;; `message-narrow-to-headers` searches for a
+	    ;; `mail-header-separator` followed by a new line. Therefore, we
+	    ;; must insert a newline if on the last line of the buffer.
+	    (when (= (point) (point-max))
+	      (insert "\n")))
 	  (progn ;; no empty line? then prepend one
 	    (goto-char (point-max))
 	    (insert "\n" sepa))))))
@@ -277,16 +325,15 @@ never hits the disk. Also see `mu4e~draft-insert-mail-header-separator."
       (let ((inhibit-read-only t))
 	(replace-match "")))))
 
-
-(defun mu4e~draft-user-wants-reply-all (origmsg)
+(defun mu4e~draft-reply-all-p (origmsg)
   "Ask user whether she wants to reply to *all* recipients.
 If there is just one recipient of ORIGMSG do nothing."
   (let* ((recipnum
 	   (+ (length (mu4e~draft-create-to-lst origmsg))
 	     (length (mu4e~draft-create-cc-lst origmsg t))))
 	  (response
-	    (if (= recipnum 1)
-	      'all ;; with one recipient, we can reply to 'all'....
+	    (if (< recipnum 2)
+	      'all ;; with less than 2 recipients, we can reply to 'all'
 	      (mu4e-read-option
 		"Reply to "
 		`( (,(format "all %d recipients" recipnum) . all)
@@ -298,21 +345,27 @@ If there is just one recipient of ORIGMSG do nothing."
 It looks something like
   <time>-<random>.<hostname>:2,
 You can append flags."
-  (let* ((hostname
-	   (downcase
-	     (save-match-data
-	       (substring system-name
-		 (string-match "^[^.]+" system-name) (match-end 0))))))
-    (format "%s-%02x%04x-%s:2,%s"
-      (format-time-string "%Y%m%d" (current-time))
-      (random 255) (random 65535) hostname (or flagstr ""))))
- 
+  (let* ((sysname (if (fboundp 'system-name)
+		    (system-name)
+		    (with-no-warnings system-name)))
+	  (sysname (if (string= sysname "") "localhost" sysname))
+	  (hostname (downcase
+		    (save-match-data
+		      (substring sysname
+				 (string-match "^[^.]+" sysname)
+				 (match-end 0))))))
+    (format "%s.%04x%04x%04x%04x.%s:2,%s"
+      (format-time-string "%s" (current-time))
+      (random 65535) (random 65535) (random 65535) (random 65535)
+      hostname (or flagstr ""))))
+
 (defun mu4e~draft-common-construct ()
   "Construct the common headers for each message."
-  (mu4e~draft-header "User-agent" (mu4e~draft-user-agent-construct))
-  (when mu4e-compose-auto-include-date
-    (mu4e~draft-header "Date" (message-make-date))))
-
+  (concat
+    (when mu4e-user-agent-string
+      (mu4e~draft-header "User-agent" mu4e-user-agent-string))
+   (when mu4e-compose-auto-include-date
+     (mu4e~draft-header "Date" (message-make-date)))))
 
 (defconst mu4e~draft-reply-prefix "Re: "
   "String to prefix replies with.")
@@ -326,7 +379,7 @@ fields will be the same as in the original."
 	     (+ (length (mu4e~draft-create-to-lst origmsg))
 	       (length (mu4e~draft-create-cc-lst origmsg t))))
 	  ;; reply-to-self implies reply-all
-	  (reply-all (or reply-to-self (mu4e~draft-user-wants-reply-all origmsg)))
+	  (reply-all (or reply-to-self (mu4e~draft-reply-all-p origmsg)))
 	  (old-msgid (plist-get origmsg :message-id))
 	  (subject
 	    (concat mu4e~draft-reply-prefix
@@ -341,11 +394,12 @@ fields will be the same as in the original."
 	  (mu4e~draft-header "To" (mu4e~draft-recipients-list-to-string
 				    (mu4e-message-field origmsg :to)))
 	  (mu4e~draft-header "Cc" (mu4e~draft-recipients-list-to-string
-				    (mu4e-message-field origmsg :cc)))) 
-	
+				    (mu4e-message-field origmsg :cc))))
+
 	;; if there's no-one in To, copy the CC-list
 	(if (zerop (length (mu4e~draft-create-to-lst origmsg)))
-	  (mu4e~draft-header "To" (mu4e~draft-recipients-construct :cc origmsg reply-all))
+	  (mu4e~draft-header "To" (mu4e~draft-recipients-construct
+				    :cc origmsg reply-all))
 	  ;; otherwise...
 	  (concat
 	    (mu4e~draft-header "To" (mu4e~draft-recipients-construct :to origmsg))
@@ -381,8 +435,10 @@ fields will be the same as in the original."
 	    ""
 	    mu4e~draft-forward-prefix)
 	  subject))
-      "\n\n"
-      (mu4e~draft-cite-original origmsg))))
+      (unless mu4e-compose-forward-as-attachment
+	(concat
+	 "\n\n"
+	  (mu4e~draft-cite-original origmsg))))))
 
 (defun mu4e~draft-newmsg-construct ()
   "Create a new message."
@@ -397,54 +453,78 @@ fields will be the same as in the original."
   "The drafts-folder for this compose buffer, based on
 `mu4e-drafts-folder', which is evaluated once.")
 
-(defun mu4e-draft-open (compose-type &optional msg)
-  "Open a draft file for a new message (when COMPOSE-TYPE is reply, forward or new),
-or open an existing draft (when COMPOSE-TYPE is edit).
+(defun mu4e~draft-open-file (path)
+  "Open the the draft file at PATH."
+  (if mu4e-compose-in-new-frame
+    (find-file-other-frame path)
+    (find-file path)))
 
-The name of the draft folder is constructed from the concatenation
-of `mu4e-maildir' and `mu4e-drafts-folder' (the latter will be
-evaluated). The message file name is a unique name determined by
-`mu4e-send-draft-file-name'. The initial contents will be created
-from either `mu4e~draft-reply-construct', or
+(defun mu4e~draft-determine-path (draft-dir)
+  "Determine the path for a new draft file."
+  (format "%s/%s/cur/%s"
+    mu4e-maildir draft-dir (mu4e~draft-message-filename-construct "DS")))
+
+
+(defun mu4e-draft-open (compose-type &optional msg)
+  "Open a draft file for a new message (when COMPOSE-TYPE is `reply',
+ `forward' or `new'), open an existing draft (when COMPOSE-TYPE
+is `edit'), or re-send an existing message (when COMPOSE-TYPE is
+`resend').
+
+The name of the draft folder is constructed from the
+concatenation of `mu4e-maildir' and `mu4e-drafts-folder' (the
+latter will be evaluated). The message file name is a unique name
+determined by `mu4e-send-draft-file-name'. The initial contents
+will be created from either `mu4e~draft-reply-construct', or
 `mu4e~draft-forward-construct' or `mu4e~draft-newmsg-construct'."
   (unless mu4e-maildir (mu4e-error "mu4e-maildir not set"))
-  (let ((draft-dir))
-    (if (eq compose-type 'edit)
-      ;; case-1: re-editing a draft messages. in this case, we do know the full
-      ;; path, but we cannot really know 'drafts folder'... we make a guess
-      (progn
+  (let ((draft-dir nil))
+    (case compose-type
+
+      (edit
+	;; case-1: re-editing a draft messages. in this case, we do know the
+	;; full path, but we cannot really know 'drafts folder'... we make a
+	;; guess
 	(setq draft-dir (mu4e~guess-maildir (mu4e-message-field msg :path)))
-	(find-file (mu4e-message-field msg :path)))
-      ;; case-2: creating a new message; in this case, we can determing
-      ;; mu4e-get-drafts-folder
-      (progn
+	(mu4e~draft-open-file (mu4e-message-field msg :path)))
+
+      (resend
+	;; case-2: copy some exisisting message to a draft message, then edit
+	;; that.
+	(setq draft-dir (mu4e~guess-maildir (mu4e-message-field msg :path)))
+	(let ((draft-path (mu4e~draft-determine-path draft-dir)))
+	  (copy-file (mu4e-message-field msg :path) draft-path)
+	  (mu4e~draft-open-file draft-path)))
+
+      ((reply forward new)
+	;; case-3: creating a new message; in this case, we can determine
+	;; mu4e-get-drafts-folder
 	(setq draft-dir (mu4e-get-drafts-folder msg))
-	(let ((draft-path
-		(format "%s/%s/cur/%s"
-		  mu4e-maildir
-		  draft-dir
-		  (mu4e~draft-message-filename-construct "DS"))))
-	  (find-file draft-path))
-	(insert
-	  (case compose-type
-	    (reply   (mu4e~draft-reply-construct msg))
-	    (forward (mu4e~draft-forward-construct msg))
-	    (new     (mu4e~draft-newmsg-construct))
-	    (t (mu4e-error "unsupported compose-type %S" compose-type))))
-	;; include the message signature (if it's set)
-	(if mu4e-compose-signature-auto-include
-	  (let ((message-signature (or mu4e-compose-signature "\n"))
-		 (message-signature-insert-empty-line t))
-	    (save-excursion
-	      (message-insert-signature)
-	      (mu4e~fontify-signature)))
-	  (insert "\n"))))
-	  ;; evaluate mu4e~drafts-drafts-folder once, here, and use that value
-	  ;; throughout.
+	(let ((draft-path (mu4e~draft-determine-path draft-dir))
+	       (initial-contents
+		 (case compose-type
+		   (reply   (mu4e~draft-reply-construct msg))
+		   (forward (mu4e~draft-forward-construct msg))
+		   (new     (mu4e~draft-newmsg-construct)))))
+	  (mu4e~draft-open-file draft-path)
+	  (insert initial-contents)
+	  (newline)
+	  ;; include the message signature (if it's set)
+	  (if (and mu4e-compose-signature-auto-include mu4e-compose-signature)
+	    (let ((message-signature mu4e-compose-signature))
+	      (save-excursion
+		(message-insert-signature)
+		(mu4e~fontify-signature))))))
+      (t (mu4e-error "unsupported compose-type %S" compose-type)))
+    ;; if we didn't find a draft folder yet, try some default
+    (unless draft-dir
+      (setq draft-dir (mu4e-get-drafts-folder msg)))
+    ;; evaluate mu4e~drafts-drafts-folder once, here, and use that value
+    ;; throughout.
     (set (make-local-variable 'mu4e~draft-drafts-folder) draft-dir)
     (put 'mu4e~draft-drafts-folder 'permanent-local t)
     (unless mu4e~draft-drafts-folder
       (mu4e-error "failed to determine drafts folder"))))
 
- 
+
 (provide 'mu4e-draft)
